from unittest.mock import patch

import pytest
from qtpy import API_NAME
from qtpy.QtGui import QColor, QPainter
from qtpy.QtWidgets import QStyleOptionViewItem

from superqt import QColorComboBox
from superqt.combobox import _color_combobox


def test_q_color_combobox(qtbot):
    wdg = QColorComboBox()
    qtbot.addWidget(wdg)
    wdg.show()
    wdg.setUserColorsAllowed(True)

    # colors can be any argument that can be passed to QColor
    # (tuples and lists will be expanded to QColor(*color)
    COLORS = [QColor("red"), "orange", (255, 255, 0), "green", "#00F", "indigo"]
    wdg.addColors(COLORS)

    colors = [wdg.itemColor(i) for i in range(wdg.count())]
    assert colors == [
        QColor("red"),
        QColor("orange"),
        QColor("yellow"),
        QColor("green"),
        QColor("blue"),
        QColor("indigo"),
        None,  # "Add Color" item
    ]

    # as with addColors, colors will be cast to QColor when using setColors
    wdg.setCurrentColor("indigo")
    assert wdg.currentColor() == QColor("indigo")
    assert wdg.currentColorName() == "#4b0082"

    wdg.clear()
    assert wdg.count() == 1  # "Add Color" item
    wdg.setUserColorsAllowed(False)
    assert not wdg.count()

    wdg.setInvalidColorPolicy(wdg.InvalidColorPolicy.Ignore)
    wdg.setInvalidColorPolicy(2)
    wdg.setInvalidColorPolicy("Raise")
    with pytest.raises(TypeError):
        wdg.setInvalidColorPolicy(1.0)  # type: ignore

    with pytest.raises(ValueError):
        wdg.addColor("invalid")


def test_q_color_delegate(qtbot):
    wdg = QColorComboBox()
    view = wdg.view()
    delegate = wdg.itemDelegate()
    qtbot.addWidget(wdg)
    wdg.show()

    # smoke tests:
    painter = QPainter()
    option = QStyleOptionViewItem()
    index = wdg.model().index(0, 0)
    delegate.paint(painter, option, index)

    wdg.addColors(["red", "orange", "yellow"])
    view.selectAll()
    index = wdg.model().index(1, 0)
    delegate.paint(painter, option, index)


@pytest.mark.skipif(API_NAME == "PySide2", reason="hangs on CI")
def test_activated(qtbot):
    wdg = QColorComboBox()
    qtbot.addWidget(wdg)
    wdg.show()
    wdg.setUserColorsAllowed(True)

    with patch.object(_color_combobox.QColorDialog, "getColor", lambda: QColor("red")):
        wdg._on_activated(wdg.count() - 1)  # "Add Color" item
    assert wdg.currentColor() == QColor("red")

    with patch.object(_color_combobox.QColorDialog, "getColor", lambda: QColor()):
        wdg._on_activated(wdg.count() - 1)  # "Add Color" item
    assert wdg.currentColor() == QColor("red")



from unittest.mock import Mock

import pytest
from qtpy.QtCore import Qt
from qtpy.QtWidgets import QApplication, QCheckBox, QVBoxLayout, QWidget

from superqt import QToggleSwitch


def test_construction(qtbot):
    wdg = QToggleSwitch("test")
    qtbot.addWidget(wdg)
    assert wdg.text() == "test"

    parent = QWidget()
    qtbot.addWidget(parent)
    wdg = QToggleSwitch(parent)
    assert wdg.parent() is parent

    with pytest.raises(TypeError):
        QToggleSwitch(parent, "test")


def test_on_and_off(qtbot):
    wdg = QToggleSwitch()
    qtbot.addWidget(wdg)
    wdg.show()
    assert not wdg.isChecked()
    wdg.setChecked(True)
    assert wdg.isChecked()
    QApplication.processEvents()
    wdg.setChecked(False)
    assert not wdg.isChecked()
    QApplication.processEvents()
    wdg.setChecked(False)
    assert not wdg.isChecked()
    wdg.toggle()
    assert wdg.isChecked()
    wdg.toggle()
    assert not wdg.isChecked()
    wdg.click()
    assert wdg.isChecked()
    wdg.click()
    assert not wdg.isChecked()
    QApplication.processEvents()


def test_disabled(qtbot):
    wdg = QToggleSwitch()
    qtbot.addWidget(wdg)
    wdg.setDisabled(True)
    assert not wdg.isEnabled()
    initial_state = wdg.isChecked()
    qtbot.mouseClick(wdg, Qt.MouseButton.LeftButton)
    assert wdg.isChecked() == initial_state  # state should not change
    wdg.setChecked(not initial_state)
    qtbot.mouseClick(wdg, Qt.MouseButton.LeftButton)
    assert wdg.isChecked() == (not initial_state)  # state should not change


def test_get_set(qtbot):
    wdg = QToggleSwitch()
    qtbot.addWidget(wdg)
    wdg.onColor = "#ff0000"
    assert wdg.onColor.name() == "#ff0000"
    wdg.offColor = "#00ff00"
    assert wdg.offColor.name() == "#00ff00"
    wdg.handleColor = "#0000ff"
    assert wdg.handleColor.name() == "#0000ff"
    wdg.setText("new text")
    assert wdg.text() == "new text"
    wdg.switchWidth = 100
    assert wdg.switchWidth == 100
    wdg.switchHeight = 100
    assert wdg.switchHeight == 100
    wdg.handleSize = 80
    assert wdg.handleSize == 80


def test_mouse_click(qtbot):
    wdg = QToggleSwitch()
    mock = Mock()
    wdg.toggled.connect(mock)
    qtbot.addWidget(wdg)
    assert not wdg.isChecked()
    mock.assert_not_called()
    qtbot.mouseClick(wdg, Qt.MouseButton.LeftButton)
    assert wdg.isChecked()
    mock.assert_called_once_with(True)
    qtbot.mouseClick(wdg, Qt.MouseButton.LeftButton)
    assert not wdg.isChecked()


def test_signal_emission_order(qtbot):
    """Check if event emmision is same for QToggleSwitch and QCheckBox"""
    wdg = QToggleSwitch()
    emitted_from_toggleswitch = []
    wdg.toggled.connect(lambda: emitted_from_toggleswitch.append("toggled"))
    wdg.pressed.connect(lambda: emitted_from_toggleswitch.append("pressed"))
    wdg.clicked.connect(lambda: emitted_from_toggleswitch.append("clicked"))
    wdg.released.connect(lambda: emitted_from_toggleswitch.append("released"))
    qtbot.addWidget(wdg)

    checkbox = QCheckBox()
    emitted_from_checkbox = []
    checkbox.toggled.connect(lambda: emitted_from_checkbox.append("toggled"))
    checkbox.pressed.connect(lambda: emitted_from_checkbox.append("pressed"))
    checkbox.clicked.connect(lambda: emitted_from_checkbox.append("clicked"))
    checkbox.released.connect(lambda: emitted_from_checkbox.append("released"))
    qtbot.addWidget(checkbox)

    emitted_from_toggleswitch.clear()
    emitted_from_checkbox.clear()
    wdg.toggle()
    checkbox.toggle()
    assert emitted_from_toggleswitch
    assert emitted_from_toggleswitch == emitted_from_checkbox

    emitted_from_toggleswitch.clear()
    emitted_from_checkbox.clear()
    wdg.click()
    checkbox.click()
    assert emitted_from_toggleswitch
    assert emitted_from_toggleswitch == emitted_from_checkbox


def test_multiple_lines(qtbot):
    container = QWidget()
    layout = QVBoxLayout(container)
    wdg0 = QToggleSwitch("line1\nline2\nline3")
    wdg1 = QToggleSwitch("line1\nline2")
    checkbox = QCheckBox()
    layout.addWidget(wdg0)
    layout.addWidget(wdg1)
    layout.addWidget(checkbox)
    container.show()
    qtbot.addWidget(container)

    assert wdg0.text() == "line1\nline2\nline3"
    assert wdg1.text() == "line1\nline2"
    assert wdg0.sizeHint().height() > wdg1.sizeHint().height()
    assert wdg1.sizeHint().height() > checkbox.sizeHint().height()
    assert wdg0.height() > wdg1.height()
    assert wdg1.height() > checkbox.height()



from typing import Any

from qtpy.QtWidgets import QPushButton, QWidget

from superqt import QFlowLayout


def test_flow_layout(qtbot: Any) -> None:
    wdg = QWidget()
    qtbot.addWidget(wdg)

    layout = QFlowLayout(wdg)
    layout.addWidget(QPushButton("Short"))
    layout.addWidget(QPushButton("Longer"))
    layout.addWidget(QPushButton("Different text"))
    layout.addWidget(QPushButton("More text"))
    layout.addWidget(QPushButton("Even longer button text"))

    wdg.setWindowTitle("Flow Layout")
    wdg.show()

    assert layout.expandingDirections()
    assert layout.heightForWidth(200) > layout.heightForWidth(400)
    assert layout.count() == 5
    assert layout.itemAt(0).widget().text() == "Short"
    layout.takeAt(0)
    assert layout.count() == 4



from typing import TYPE_CHECKING

import pytest
from qtpy.QtGui import QIcon
from qtpy.QtWidgets import QPushButton

from superqt import QIconifyIcon

if TYPE_CHECKING:
    from pytestqt.qtbot import QtBot


def test_qiconify(qtbot: "QtBot", monkeypatch: "pytest.MonkeyPatch") -> None:
    monkeypatch.setenv("PYCONIFY_CACHE", "0")
    pytest.importorskip("pyconify")

    icon = QIconifyIcon("bi:alarm-fill", color="red", flip="vertical")
    icon.addKey("bi:alarm", color="blue", rotate=90, state=QIcon.State.On)

    btn = QPushButton()
    qtbot.addWidget(btn)
    btn.setIcon(icon)
    btn.show()



import sys
from enum import Enum, Flag, IntEnum, IntFlag

import pytest

from superqt.combobox import QEnumComboBox
from superqt.combobox._enum_combobox import NONE_STRING


class Enum1(Enum):
    a = 1
    b = 2
    c = 3

    ALIAS = a


class Enum2(Enum):
    d = 1
    e = 2
    f = 3
    g = 4


class Enum3(Enum):
    a = 1
    b = 2
    c = 3

    def __str__(self):
        return self.name + "1"


class Enum4(Enum):
    a_1 = 1
    b_2 = 2
    c_3 = 3


class IntEnum1(IntEnum):
    a = 1
    b = 2
    c = 5


class IntFlag1(IntFlag):
    a = 1
    b = 2
    c = 4


class Flag1(Flag):
    a = 1
    b = 2
    c = 4


class IntFlag2(IntFlag):
    a = 1
    b = 2
    c = 3


class Flag2(IntFlag):
    a = 1
    b = 2
    c = 5


class FlagOrNum(IntFlag):
    a = 3
    b = 5
    c = 8


def test_simple_create(qtbot):
    enum = QEnumComboBox(enum_class=Enum1)
    qtbot.addWidget(enum)
    assert enum.count() == 3
    assert [enum.itemText(i) for i in range(enum.count())] == ["a", "b", "c"]


def test_simple_create2(qtbot):
    enum = QEnumComboBox()
    qtbot.addWidget(enum)
    assert enum.count() == 0
    enum.setEnumClass(Enum1)
    assert enum.count() == 3
    assert [enum.itemText(i) for i in range(enum.count())] == ["a", "b", "c"]


def test_replace(qtbot):
    enum = QEnumComboBox(enum_class=Enum1)
    qtbot.addWidget(enum)
    assert enum.count() == 3
    assert enum.enumClass() == Enum1
    assert isinstance(enum.currentEnum(), Enum1)
    enum.setEnumClass(Enum2)
    assert enum.enumClass() == Enum2
    assert isinstance(enum.currentEnum(), Enum2)
    assert enum.count() == 4
    assert [enum.itemText(i) for i in range(enum.count())] == ["d", "e", "f", "g"]


def test_str_replace(qtbot):
    enum = QEnumComboBox(enum_class=Enum3)
    qtbot.addWidget(enum)
    assert enum.count() == 3
    assert [enum.itemText(i) for i in range(enum.count())] == ["a1", "b1", "c1"]


def test_underscore_replace(qtbot):
    enum = QEnumComboBox(enum_class=Enum4)
    qtbot.addWidget(enum)
    assert enum.count() == 3
    assert [enum.itemText(i) for i in range(enum.count())] == ["a 1", "b 2", "c 3"]


def test_change_value(qtbot):
    enum = QEnumComboBox(enum_class=Enum1)
    qtbot.addWidget(enum)
    assert enum.currentEnum() == Enum1.a
    with qtbot.waitSignal(
        enum.currentEnumChanged, check_params_cb=lambda x: isinstance(x, Enum)
    ):
        enum.setCurrentEnum(Enum1.c)
    assert enum.currentEnum() == Enum1.c


def test_no_enum(qtbot):
    enum = QEnumComboBox()
    assert enum.enumClass() is None
    qtbot.addWidget(enum)
    assert enum.currentEnum() is None


def test_prohibited_methods(qtbot):
    enum = QEnumComboBox(enum_class=Enum1)
    qtbot.addWidget(enum)
    with pytest.raises(RuntimeError):
        enum.addItem("aaa")
    with pytest.raises(RuntimeError):
        enum.addItems(["aaa", "bbb"])
    with pytest.raises(RuntimeError):
        enum.insertItem(0, "aaa")
    with pytest.raises(RuntimeError):
        enum.insertItems(0, ["aaa", "bbb"])
    assert enum.count() == 3


def test_optional(qtbot):
    enum = QEnumComboBox(enum_class=Enum1, allow_none=True)
    qtbot.addWidget(enum)
    assert [enum.itemText(i) for i in range(enum.count())] == [
        NONE_STRING,
        "a",
        "b",
        "c",
    ]
    assert enum.currentText() == NONE_STRING
    assert enum.currentEnum() is None
    enum.setCurrentEnum(Enum1.a)
    assert enum.currentText() == "a"
    assert enum.currentEnum() == Enum1.a
    assert enum.enumClass() is Enum1
    enum.setCurrentEnum(None)
    assert enum.currentText() == NONE_STRING
    assert enum.currentEnum() is None


def test_simple_create_int_enum(qtbot):
    enum = QEnumComboBox(enum_class=IntEnum1)
    qtbot.addWidget(enum)
    assert [enum.itemText(i) for i in range(enum.count())] == ["a", "b", "c"]


@pytest.mark.parametrize("enum_class", [IntFlag1, Flag1])
def test_enum_flag_create(qtbot, enum_class):
    enum = QEnumComboBox(enum_class=enum_class)
    qtbot.addWidget(enum)
    assert [enum.itemText(i) for i in range(enum.count())] == [
        "a",
        "b",
        "c",
        "a|b",
        "a|c",
        "b|c",
        "a|b|c",
    ]
    enum.setCurrentText("a|b")
    assert enum.currentEnum() == enum_class.a | enum_class.b


def test_enum_flag_create_collision(qtbot):
    enum = QEnumComboBox(enum_class=IntFlag2)
    qtbot.addWidget(enum)
    assert [enum.itemText(i) for i in range(enum.count())] == ["a", "b", "c"]


@pytest.mark.skipif(
    sys.version_info >= (3, 11), reason="different representation in 3.11"
)
def test_enum_flag_create_collision_evaluated_to_seven(qtbot):
    enum = QEnumComboBox(enum_class=FlagOrNum)
    qtbot.addWidget(enum)
    assert [enum.itemText(i) for i in range(enum.count())] == [
        "a",
        "b",
        "c",
        "a|b",
        "a|c",
        "b|c",
        "a|b|c",
    ]


@pytest.mark.skipif(
    sys.version_info < (3, 11), reason="StrEnum is introduced in python 3.11"
)
def test_create_str_enum(qtbot):
    from enum import StrEnum

    class StrEnum1(StrEnum):
        a = "a"
        b = "b"
        c = "c"

    enum = QEnumComboBox(enum_class=StrEnum1)
    qtbot.addWidget(enum)
    assert [enum.itemText(i) for i in range(enum.count())] == ["a", "b", "c"]
